# simulateMethyData.R
rm(list=ls())

Args <- commandArgs(TRUE);

coordFolder = Args[1];
samplefile = Args[2];
selected_sites_prefix = Args[3];

set.seed(1);

# simulate a small data set
N = 50; # number of subjects
pChr = rpois(22, 100);

# The following paramter is used in the paper
#N = 1000; # number of subjects
#pChr = rpois(22, 4000);

p = sum(pChr); # number of CpG blocks
sampleIDs = matrix("", N, 1);
for (i in 1:N){
    sampleIDs[i] = paste("01ABCD", sprintf("%04d", i), sep="");
}


########### simulation data #############
#set.seed(1);
caseControlStatus = c(rep(1, N/2), rep(0, N/2));
nfactor_cont = 2; #as.integer(Args[4])# 2; # number of continuous confounding factors

K = 5; # number of confounding factors;
X = matrix(0, N, p);
# base level
methBaseLevel = c(0:100)
for (j in 1:p){
    X[,j] = rnorm(N, sample(methBaseLevel, 1), 40) ;
}

# continuous factors
factors_continuous = matrix(rnorm(N*K, 0, 10), N, K);
betak = c(4, 4, 4, 4, 4);
for (l in 1:K){
    factors_continuous[,l] = factors_continuous[,l] + betak[l]*caseControlStatus;
}

# dichotomous factors
factors_binary = matrix(0, N, K);
p1 = 0.6;
p2 = 0.4;
for (l in 1:K){
   factors_binary[1:(N/2),l] = sample(c(1,0), N/2, replace=T, prob = c(p1, 1-p1));
   factors_binary[(N/2+1):N,l] = sample(c(1,0), N/2, replace=T, prob = c(p2, 1-p2));
}
factors_binary = factors_binary*20;

if(nfactor_cont>0 && nfactor_cont<K){
   factors = cbind(factors_continuous[, 1:nfactor_cont], factors_binary[, 1:(K-nfactor_cont)]);
} else if(nfactor_cont==0){
   factors = factors_binary;
} else if(nfactor_cont==K){
   factors = factors_continuous;
}

factorsLoc = matrix(-1, 2, K);
for (l in 1:K){
    startChr = 1 + (l-1)*2;
    endChr = startChr+2;
    if(startChr>1){
    	startCol = sum(pChr[1:(startChr-1)]) + 1; 
    } else {
    	startCol = 1;
    }
    endCol = sum(pChr[1:endChr]);
    factorsLoc[, l] = c(startCol, endCol);
    X[, startCol:endCol] = X[, startCol:endCol] + factors[,l]; 
}

y = caseControlStatus;

############## end of simulation data ##############


# to avoid negative numbers
X = X - min(c(X));

#print(cor(X[,1:10], X[,1:10]));


# output sample ID file
idFile = samplefile;

dirPath = dirname(idFile);
if(!file.exists(dirPath)){
		dir.create(dirPath, recursive=T);
}
file.create(idFile);

# simulate some covariates in association test
sex = rbinom(N, 1, 0.5);
age = rpois(N, 35);

# simulate lab covariates
c1 = rnorm(N, 4000, 100);
c2 = rbinom(N, 1, 0.5);


phenoData = data.frame(sampleIDs, y, c1, c2, sex, age, stringsAsFactors=F);
names(phenoData) = c("sampleID", "outcome", "c1", "c2", "sex", "age");
write.csv(phenoData, file = idFile, row.names=F, quote=F);

# randomly generate selected sites
startPos = 1;
for (i in 1:length(pChr)){
    	chrFile = paste(selected_sites_prefix, i, ".csv", sep="");
    	dirPath = dirname(chrFile);
    	if(!file.exists(dirPath)){
    		dir.create(dirPath, recursive=T);
    	}
    	numberOfCites = pChr[i];
    
	selectionData = rbinom(numberOfCites, 1, prob=1);  # select all cites for simulation
	fullData = cbind(1:numberOfCites, selectionData);
	colnames(fullData) = c("coord", "select");

	write.csv(fullData, file=chrFile, quote=F, row.names=F);
    
    	startPos = startPos  + numberOfCites;
}

# output sample coverage data
startPos = 1;
for (i in 1:length(pChr)){
    chrFolder = paste(coordFolder, "/chr", i, sep="");
    dir.create(chrFolder, recursive=T);
    numberOfCites = pChr[i];
    for (j in 1:N){
        coverageFile = paste(chrFolder, "/", sampleIDs[j], ".csv", sep="");
        coverageData = X[j, startPos:(startPos + numberOfCites - 1)];
        coverageData = cbind(1:numberOfCites, coverageData);
        colnames(coverageData) = c("coord", "coverage");
        write.csv(coverageData, file=coverageFile, quote=F, row.names=F);
    }
    startPos = startPos  + numberOfCites;
}

#warnings()