# validation.R
# compare the result calculated by the C/R code and the pure R code

rm(list=ls())


Args <- commandArgs(TRUE);
dataFolder = Args[1];
samplefile = Args[2];
pcaResultsDir = Args[3];
chr_begin = as.integer(Args[4]);
chr_end = as.integer(Args[5]);
if(length(Args)>5){
	covFile = Args[6];
}


sampleIDsFile = paste(samplefile, sep="");
sampleIDs = as.matrix(read.table(sampleIDsFile, colClasses="character"));
N = length(sampleIDs)

startPos = 1;
for (i in chr_begin:chr_end){
    chrFolder = paste(dataFolder, "/chr", i, sep="");
    for (j in 1:N){
        coverageFile = paste(chrFolder, "/", sampleIDs[j], ".csv", sep="");
        coverageData= read.csv(coverageFile);
        coverageData = as.matrix(coverageData[,2]);
        if(j == 1){
            numberOfCites = dim(coverageData)[1];
        } else if(numberOfCites !=dim(coverageData)[1]){
            print(paste("Error: number of cites are different from file: ", coverageFile));
        }
        if(j == 1){
            XFromFile = matrix(NA, N, numberOfCites);
        }
        XFromFile[j,] = c(coverageData);
    }
    if(i == 1){
        XFromFileAllChr = XFromFile;
    } else {
        XFromFileAllChr = cbind(XFromFileAllChr, XFromFile);
    }
    
    startPos = startPos  + numberOfCites;
}

X = XFromFileAllChr;

if(length(Args)>5){
	# regress out lab covariates
	covFile = paste(covFile, sep="");
	covData = read.csv(covFile, as.is=T);
	covariates = as.matrix(covData[,2:dim(covData)[2]]);
	p = dim(X)[2];
	Xreg = X;
	for (i in 1:p){
		Xreg[,i] = residuals(lm(X[,i]~covariates));
	}

	X = Xreg;
	#dataFolder = paste(dataFolder, "/", "regressOutCovariate", sep="");
}


# calculate mean and SD
#meanX = colMeans(X);
#sdX = sd(X);
#print(sdX[1:10]);
#
# calculate XXT and XXT after standardization
#Xcentered = t(t(X) - meanX);
#Xstd = t(t(Xcentered)/sdX);

#XXT = Xcentered %*% t(Xcentered);
#XXTstd = Xstd %*% t(Xstd);

# Do PCA here purely in R
# using covariance matrix
rsCov = prcomp(X, retx = T);
PCScoresCov = rsCov$x;
eigenvaluesCov = rsCov$sdev^2;
eigenvectorsCov = rsCov$rotation;

# compare the results from C/R coding and the pure R coding


cPCScoresCov = read.csv(paste(pcaResultsDir, "/cov_scores.csv", sep=""), header=T);
cPCScoresCov = as.matrix(cPCScoresCov[, 2:dim(cPCScoresCov)[2]]);
ceigenvaluesCov = read.csv(paste(pcaResultsDir, "/cov_eigenval.csv", sep=""));

print("norm of differences in eigenvalues: cov")
print(norm(t(ceigenvaluesCov - eigenvaluesCov), "F")); # eigenvalues
print("norm of differences in PC scores: cov")
nPCs = min(dim(PCScoresCov)[2], dim(cPCScoresCov)[2]); # PC scores
nPCs = min(nPCs, sum(eigenvaluesCov>1e-11));
print(norm(abs(PCScoresCov[,1:nPCs]) - abs(cPCScoresCov[,1:nPCs]), "F"));

# read eigenvectors
startPos = 1;
p = dim(X)[2];
ceigenvectorsCov = matrix(NA, p, nPCs);
for (i in chr_begin:chr_end){
    chrFolder = paste(pcaResultsDir, "/loadings", sep="");
    loadingFile = paste(chrFolder, "/", "cov_loadings_chr", i, ".csv", sep="");
    loadingData= read.csv(loadingFile);
    loadingData = loadingData[,2:dim(loadingData)[2]];
    numberOfCites = dim(loadingData)[1];
    ceigenvectorsCov[startPos:(startPos + numberOfCites - 1), 1:dim(loadingData)[2]] = as.matrix(loadingData);
    startPos = startPos  + numberOfCites;
    if(i == 1) {nDim = dim(loadingData)[2];}
}

print("norm of differences in loading: cov")
print(norm(abs(eigenvectorsCov[,1:nDim]) - abs(ceigenvectorsCov[,1:nDim]), "F"));


# using correlation matrix
rsCor = prcomp(X, retx = T, scale=T);
PCScoresCor = rsCor$x;
eigenvaluesCor = rsCor$sdev^2;
eigenvectorsCor = rsCor$rotation;

# compare the results from C/R coding and the pure R coding
cPCScoresCor = read.csv(paste(pcaResultsDir, "/cor_scores.csv", sep=""), header=T);
cPCScoresCor = as.matrix(cPCScoresCor[, 2:dim(cPCScoresCor)[2]]);
ceigenvaluesCor = read.csv(paste(pcaResultsDir, "/cor_eigenval.csv", sep=""));

print("norm of differences in eigenvalues: cor")
print(norm(t(ceigenvaluesCor - eigenvaluesCor), "F")); # eigenvalues
print("norm of differences in PC scores: cor")
nPCs = min(dim(PCScoresCor)[2], dim(cPCScoresCor)[2]); # PC scores
nPCs = min(nPCs, sum(eigenvaluesCor>1e-11));
print(norm(abs(PCScoresCor[,1:nPCs]) - abs(cPCScoresCor[,1:nPCs]), "F"));

# read eigenvectors
startPos = 1;
ceigenvectorsCor = matrix(NA, p, nPCs);
for (i in chr_begin:chr_end){
    chrFolder = paste(pcaResultsDir, "/loadings", sep="");
    loadingFile = paste(chrFolder, "/", "cor_loadings_chr", i, ".csv", sep="");
    loadingData= read.csv(loadingFile);
    loadingData = loadingData[,2:dim(loadingData)[2]];
    numberOfCites = dim(loadingData)[1];
    ceigenvectorsCor[startPos:(startPos + numberOfCites - 1), 1:dim(loadingData)[2]] = as.matrix(loadingData);
    startPos = startPos  + numberOfCites;
    if(i == 1) {nDim = dim(loadingData)[2];}
}

print("norm of differences in loading: cor")
print(norm(abs(eigenvectorsCor[,1:nDim]) - abs(ceigenvectorsCor[,1:nDim]), "F"));

